var marked = require('marked');
var renderer = new marked.Renderer();
var toURL = require('./transformer-urlizetags.js').toURL;
var highlightjs = require('highlight.js');
var path = require('path');
var minificationOK = true;
try {require('lwip');}
catch(e) {
  minificationOK = false;
}

var images = {};

renderer.heading = function(text, level, raw) {
  return '<h'
    + (level+2)
    + ' id="'
    + this.options.headerPrefix
    + toURL(raw)
    + '">'
    + text
    + '</h'
    + (level+2)
    + '>\n';
}

renderer.paragraph = function(text) {
  if(text.match('\\<img')) return '<div class="has-image">'+text+'</div>\n';
  else return '<p>'+text+'</p>\n';
}

marked.setOptions({
  renderer: renderer,
  gfm: true,
  tables: true,
  breaks: false,
  pedantic: false,
  sanitize: false,
  smartLists: true,
  smartypants: false,
});

module.exports = function() {
  var tr = this;
  tr.fancyname = "transformer markdown";
  tr.pre = function(cb, list) {
    list.images.forEach((img) => {
      images[img.file] = img;
    });
    console.log(images);
    cb();
  };
  tr.forEachPage = function(article, cb) {
    article.origContent = article.content;
    renderer.image = function(href, title, text) {
      href = href.split(path.sep).join("/");
      href = href.split(" =");
      var size = href[1];
      href = href[0];
      var rel = path.relative(process.cwd(),path.resolve(article.file,href));
      rel = rel.split(path.sep).join("/");
      var img = images[rel];
      var full;
      if(img) {
        if(img.minified) {
          full = href;
          href = path.relative(article.file, img.thumb);
        }else if(!minificationOK) {
          full = href;
        }
      }
      else if(process.argv[3] !== "final") {
        if(!href.match("://")) {
          href = "https://ok1kvk.cz"+href;
        }
      }

      var out = '<img src="' + href + '" alt="' + text + '"';
      if(size !== undefined) {
        size = size.split('x');
        if(size[0].length > 0) {
          out += ' width="' + size[0] + '"';
        }
        if(size.length > 1 && size[1].length > 0) {
          out += ' height="' + size[1] + '"';
        }
      }
      if (title) {
        out += ' title="' + title + '"';
      }
      if(!minificationOK) {
        out += ' style="max-width:512px"';
      }
      out += '>';
      if(full) {
        var a = '<a href="'+full+'"';
        a += ' data-lightbox="group"';
        a += ' data-title="' + text + '"';
        a += '>';
        out = a+out+'</a>';
      }

      return out;
    };

    article.content = marked(article.content, {
      highlight: function (code) {
        var h;
        if(highlightjs.getLanguage(article.metadata.lang))
          h = highlightjs.highlight(article.metadata.lang, code, true);
        else
          h = highlightjs.highlightAuto(code);
        return h.value;
      },
      renderer: renderer
    });
    cb();
  }
}
